// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package google.protobuf.python.internal;

import "google/protobuf/descriptor.proto";

message FooOptions {
  optional string foo_name = 1;
}

extend .google.protobuf.FileOptions {
  optional FooOptions foo_options = 120436268;
}
