// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package field_presence_test;

import "google/protobuf/unittest.proto";

option java_package = "com.google.protobuf";
option java_outer_classname = "FieldPresenceTestProto";

message TestAllTypes {
  enum NestedEnum {
    FOO = 0;
    BAR = 1;
    BAZ = 2;
  }
  message NestedMessage {
    int32 value = 1;
  }

  int32 optional_int32 = 1;
  string optional_string = 2;
  bytes optional_bytes = 3;
  NestedEnum optional_nested_enum = 4;
  NestedMessage optional_nested_message = 5;
  protobuf_unittest.TestRequired optional_proto2_message = 6;
  NestedMessage optional_lazy_message = 7 [lazy = true];

  oneof oneof_field {
    int32 oneof_int32 = 11;
    uint32 oneof_uint32 = 12;
    string oneof_string = 13;
    bytes oneof_bytes = 14;
    NestedEnum oneof_nested_enum = 15;
    NestedMessage oneof_nested_message = 16;
    protobuf_unittest.TestRequired oneof_proto2_message = 17;
  }

  repeated int32 repeated_int32 = 21;
  repeated string repeated_string = 22;
  repeated bytes repeated_bytes = 23;
  repeated NestedEnum repeated_nested_enum = 24;
  repeated NestedMessage repeated_nested_message = 25;
  repeated protobuf_unittest.TestRequired repeated_proto2_message = 26;
  repeated NestedEnum packed_nested_enum = 27 [packed = true];
}

message TestOptionalFieldsOnly {
  int32 optional_int32 = 1;
  string optional_string = 2;
  bytes optional_bytes = 3;
  TestAllTypes.NestedEnum optional_nested_enum = 4;
  TestAllTypes.NestedMessage optional_nested_message = 5;
  protobuf_unittest.TestRequired optional_proto2_message = 6;
  TestAllTypes.NestedMessage optional_lazy_message = 7 [lazy = true];
}

message TestRepeatedFieldsOnly {
  repeated int32 repeated_int32 = 21;
  repeated string repeated_string = 22;
  repeated bytes repeated_bytes = 23;
  repeated TestAllTypes.NestedEnum repeated_nested_enum = 24;
  repeated TestAllTypes.NestedMessage repeated_nested_message = 25;
  repeated protobuf_unittest.TestRequired repeated_proto2_message = 26;
}
