#region Copyright notice and license
// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd
#endregion

namespace Google.Protobuf.WellKnownTypes
{
    public static partial class WrappersReflection
    {
        /// <summary>
        /// Field number for the single "value" field in all wrapper types.
        /// </summary>
        internal const int WrapperValueFieldNumber = Int32Value.ValueFieldNumber;
    }
}
