syntax = "proto3";

package upper_enum_value;

enum NotAllowed {
  ABSTRACT = 0;
  AND = 1;
  ARRAY = 2;
  AS = 3;
  BREAK = 4;
  CALLABLE = 5;
  CASE     = 6;
  CATCH    = 7;
  CLASS    = 8;
  CLONE    = 9;
  CONST    = 10;
  CONTINUE = 11;
  DECLARE  = 12;
  DEFAULT  = 13;
  DIE      = 14;
  DO       = 15;
  ECHO     = 16;
  ELSE     = 17;
  ELSEIF   = 18;
  EMPTY    = 19;
  ENDDECLARE = 20;
  ENDFOR     = 21;
  ENDFOREACH = 22;
  ENDIF      = 23;
  ENDSWITCH = 24;
  ENDWHILE = 25;
  EVAL = 26;
  EXIT = 27;
  EXTENDS = 28;
  FINAL = 29;
  FINALLY = 30;
  FOR = 31;
  FOREACH = 32;
  FUNCTION = 33;
  FN = 34;
  GLOBAL = 35;
  GOTO = 36;
  IF = 37;
  IMPLEMENTS = 38;
  INCLUDE = 39;
  INCLUDE_ONCE = 40;
  INSTANCEOF = 41;
  INSTEADOF = 42;
  INTERFACE = 43;
  ISSET = 44;
  LIST = 45;
  MATCH = 46;
  NAMESPACE = 47;
  NEW = 48;
  OR = 49;
  PARENT = 77;
  PRINT = 50;
  PRIVATE = 51;
  PROTECTED = 52;
  PUBLIC = 53;
  READONLY = 79;
  REQUIRE = 54;
  REQUIRE_ONCE = 55;
  RETURN = 56;
  SELF = 78;
  STATIC = 57;
  SWITCH = 58;
  THROW = 59;
  TRAIT = 60;
  TRY = 61;
  UNSET = 62;
  USE = 63;
  VAR = 64;
  WHILE = 65;
  XOR = 66;
  YIELD = 67;
  INT = 68;
  FLOAT = 69;
  BOOL = 70;
  STRING = 71;
  TRUE = 72;
  FALSE = 73;
  NULL = 74;
  VOID = 75;
  ITERABLE = 76;
}
