// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#include "google/protobuf/unittest_proto3.pb.h"

#define LITE_TEST_NAME Proto3LiteTest
#define UNITTEST ::proto3_unittest

// Must include after the above macros.
#include "google/protobuf/proto3_lite_unittest.inc"

// Make extract script happy.
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
