// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package any_test;

import "google/protobuf/any.proto";

option java_package = "any_test";
option java_outer_classname = "AnyTestProto";

message TestAny {
  google.protobuf.Any value = 1;
}
