// NOLINT(no_package_specified): Test proto with no package

// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

enum NoPackageEnum {
  NO_PACKAGE_VALUE_0 = 0;
  NO_PACKAGE_VALUE_1 = 1;
}

message NoPackageMessage {
  optional NoPackageEnum no_package_enum = 1;
}
