// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package wrappers_test;

import "google/protobuf/wrappers.proto";

option java_package = "com.google.protobuf.wrapperstest";
option java_outer_classname = "WrappersTestProto";

message TopLevelMessage {
  int32 field1 = 1;
  .google.protobuf.DoubleValue field_double = 2;
  .google.protobuf.FloatValue field_float = 3;
  .google.protobuf.Int64Value field_int64 = 4;
  .google.protobuf.UInt64Value field_uint64 = 5;
  .google.protobuf.Int32Value field_int32 = 6;
  .google.protobuf.UInt32Value field_uint32 = 7;
  .google.protobuf.BoolValue field_bool = 8;
  .google.protobuf.StringValue field_string = 9;
  .google.protobuf.BytesValue field_bytes = 10;
}
