// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: jieluo@google.com (Jie Luo)

syntax = "proto2";

package google.protobuf.internal;

import "google/protobuf/any.proto";

message TestAny {
  optional google.protobuf.Any value = 1;
  optional int32 int_value = 2;
  map<string, int32> map_value = 3;
  extensions 10 to max;
}

message TestAnyExtension1 {
  extend TestAny {
    optional TestAnyExtension1 extension1 = 98418603;
  }
  optional int32 i = 15;
}
