syntax = "proto3";

package upper_enum;

enum ABSTRACT { ZERO1 = 0; }
enum AND { ZERO2 = 0; }
enum ARRAY { ZERO3 = 0; }
enum AS { ZERO4 = 0; }
enum BREAK { ZERO5 = 0; }
enum CALLABLE { ZERO6 = 0; }
enum CASE { ZERO7 = 0; }
enum CATCH { ZERO8 = 0; }
enum CLASS { ZERO9 = 0; }
enum CLONE { ZERO10 = 0; }
enum CONST { ZERO11 = 0; }
enum CONTINUE { ZERO12 = 0; }
enum DECLARE { ZERO13 = 0; }
enum DEFAULT { ZERO14 = 0; }
enum DIE { ZERO15 = 0; }
enum DO { ZERO16 = 0; }
enum ECHO { ZERO17 = 0; }
enum ELSE { ZERO18 = 0; }
enum ELSEIF { ZERO19 = 0; }
enum EMPTY { ZERO20 = 0; }
enum ENDDECLARE { ZERO21 = 0; }
enum ENDFOR { ZERO22 = 0; }
enum ENDFOREACH { ZERO23 = 0; }
enum ENDIF { ZERO24 = 0; }
enum ENDSWITCH { ZERO25 = 0; }
enum ENDWHILE { ZERO26 = 0; }
enum EVAL { ZERO27 = 0; }
enum EXIT { ZERO28 = 0; }
enum EXTENDS { ZERO29 = 0; }
enum FINAL { ZERO30 = 0; }
enum FINALLY { ZERO31 = 0; }
enum FN { ZERO32 = 0; }
enum FOR { ZERO33 = 0; }
enum FOREACH { ZERO34 = 0; }
enum FUNCTION { ZERO35 = 0; }
enum GLOBAL { ZERO36 = 0; }
enum GOTO { ZERO37 = 0; }
enum IF { ZERO38 = 0; }
enum IMPLEMENTS { ZERO39 = 0; }
enum INCLUDE { ZERO40 = 0; }
enum INCLUDE_ONCE { ZERO41 = 0; }
enum INSTANCEOF { ZERO42 = 0; }
enum INSTEADOF { ZERO43 = 0; }
enum INTERFACE { ZERO44 = 0; }
enum ISSET { ZERO45 = 0; }
enum LIST { ZERO46 = 0; }
enum MATCH { ZERO47 = 0; }
enum NAMESPACE { ZERO48 = 0; }
enum NEW { ZERO49 = 0; }
enum OR { ZERO50 = 0; }
enum PARENT { ZERO78 = 0; }
enum PRINT { ZERO51 = 0; }
enum PRIVATE { ZERO52 = 0; }
enum PROTECTED { ZERO53 = 0; }
enum PUBLIC { ZERO54 = 0; }
enum READONLY { ZERO80 = 0; }
enum REQUIRE { ZERO55 = 0; }
enum REQUIRE_ONCE { ZERO56 = 0; }
enum RETURN { ZERO57 = 0; }
enum SELF { ZERO79 = 0; }
enum STATIC { ZERO58 = 0; }
enum SWITCH { ZERO59 = 0; }
enum THROW { ZERO60 = 0; }
enum TRAIT { ZERO61 = 0; }
enum TRY { ZERO62 = 0; }
enum UNSET { ZERO63 = 0; }
enum USE { ZERO64 = 0; }
enum VAR { ZERO65 = 0; }
enum WHILE { ZERO66 = 0; }
enum XOR { ZERO67 = 0; }
enum YIELD { ZERO68 = 0; }
enum INT { ZERO69 = 0; }
enum FLOAT { ZERO70 = 0; }
enum BOOL { ZERO71 = 0; }
enum STRING { ZERO72 = 0; }
enum TRUE { ZERO73 = 0; }
enum FALSE { ZERO74 = 0; }
enum NULL { ZERO75 = 0; }
enum VOID { ZERO76 = 0; }
enum ITERABLE { ZERO77 = 0; }
