/*
 * Protocol Buffers - Google's data interchange format
 * Copyright 2014 Google Inc.  All rights reserved.
 * https://developers.google.com/protocol-buffers/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: sentinel.proto

package com.google.protobuf.jruby;

public final class SentinelOuterClass {
  private SentinelOuterClass() {}

  public static void registerAllExtensions(com.google.protobuf.ExtensionRegistry registry) {}

  public interface SentinelOrBuilder
      extends
      // @@protoc_insertion_point(interface_extends:com.google.protobuf.jruby.Sentinel)
      com.google.protobuf.MessageOrBuilder {

    /** <code>optional int32 default_int32 = 1;</code> */
    int getDefaultInt32();

    /** <code>optional int64 default_int64 = 2;</code> */
    long getDefaultInt64();

    /** <code>optional uint32 default_unit32 = 3;</code> */
    int getDefaultUnit32();

    /** <code>optional uint64 default_uint64 = 4;</code> */
    long getDefaultUint64();

    /** <code>optional string default_string = 5;</code> */
    java.lang.String getDefaultString();
    /** <code>optional string default_string = 5;</code> */
    com.google.protobuf.ByteString getDefaultStringBytes();

    /** <code>optional bool default_bool = 6;</code> */
    boolean getDefaultBool();

    /** <code>optional float default_float = 7;</code> */
    float getDefaultFloat();

    /** <code>optional double default_double = 8;</code> */
    double getDefaultDouble();

    /** <code>optional bytes default_bytes = 9;</code> */
    com.google.protobuf.ByteString getDefaultBytes();
  }
  /** Protobuf type {@code com.google.protobuf.jruby.Sentinel} */
  public static final class Sentinel extends com.google.protobuf.GeneratedMessage
      implements
      // @@protoc_insertion_point(message_implements:com.google.protobuf.jruby.Sentinel)
      SentinelOrBuilder {
    // Use Sentinel.newBuilder() to construct.
    private Sentinel(com.google.protobuf.GeneratedMessage.Builder builder) {
      super(builder);
    }

    private Sentinel() {
      defaultInt32_ = 0;
      defaultInt64_ = 0L;
      defaultUnit32_ = 0;
      defaultUint64_ = 0L;
      defaultString_ = "";
      defaultBool_ = false;
      defaultFloat_ = 0F;
      defaultDouble_ = 0D;
      defaultBytes_ = com.google.protobuf.ByteString.EMPTY;
    }

    @java.lang.Override
    public final com.google.protobuf.UnknownFieldSet getUnknownFields() {
      return com.google.protobuf.UnknownFieldSet.getDefaultInstance();
    }

    public static final com.google.protobuf.Descriptors.Descriptor getDescriptor() {
      return com.google.protobuf.jruby.SentinelOuterClass
          .internal_static_com_google_protobuf_jruby_Sentinel_descriptor;
    }

    protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.google.protobuf.jruby.SentinelOuterClass
          .internal_static_com_google_protobuf_jruby_Sentinel_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.google.protobuf.jruby.SentinelOuterClass.Sentinel.class,
              com.google.protobuf.jruby.SentinelOuterClass.Sentinel.Builder.class);
    }

    public static final com.google.protobuf.Parser<Sentinel> PARSER =
        new com.google.protobuf.AbstractParser<Sentinel>() {
          public Sentinel parsePartialFrom(
              com.google.protobuf.CodedInputStream input,
              com.google.protobuf.ExtensionRegistryLite extensionRegistry)
              throws com.google.protobuf.InvalidProtocolBufferException {
            Builder builder = newBuilder();
            try {
              builder.mergeFrom(input, extensionRegistry);
            } catch (com.google.protobuf.InvalidProtocolBufferException e) {
              throw e.setUnfinishedMessage(builder.buildPartial());
            } catch (java.io.IOException e) {
              throw new com.google.protobuf.InvalidProtocolBufferException(e.getMessage())
                  .setUnfinishedMessage(builder.buildPartial());
            }
            return builder.buildPartial();
          }
        };

    @java.lang.Override
    public com.google.protobuf.Parser<Sentinel> getParserForType() {
      return PARSER;
    }

    public static final int DEFAULT_INT32_FIELD_NUMBER = 1;
    private int defaultInt32_;
    /** <code>optional int32 default_int32 = 1;</code> */
    public int getDefaultInt32() {
      return defaultInt32_;
    }

    public static final int DEFAULT_INT64_FIELD_NUMBER = 2;
    private long defaultInt64_;
    /** <code>optional int64 default_int64 = 2;</code> */
    public long getDefaultInt64() {
      return defaultInt64_;
    }

    public static final int DEFAULT_UNIT32_FIELD_NUMBER = 3;
    private int defaultUnit32_;
    /** <code>optional uint32 default_unit32 = 3;</code> */
    public int getDefaultUnit32() {
      return defaultUnit32_;
    }

    public static final int DEFAULT_UINT64_FIELD_NUMBER = 4;
    private long defaultUint64_;
    /** <code>optional uint64 default_uint64 = 4;</code> */
    public long getDefaultUint64() {
      return defaultUint64_;
    }

    public static final int DEFAULT_STRING_FIELD_NUMBER = 5;
    private java.lang.Object defaultString_;
    /** <code>optional string default_string = 5;</code> */
    public java.lang.String getDefaultString() {
      java.lang.Object ref = defaultString_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        if (bs.isValidUtf8()) {
          defaultString_ = s;
        }
        return s;
      }
    }
    /** <code>optional string default_string = 5;</code> */
    public com.google.protobuf.ByteString getDefaultStringBytes() {
      java.lang.Object ref = defaultString_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b =
            com.google.protobuf.ByteString.copyFromUtf8((java.lang.String) ref);
        defaultString_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int DEFAULT_BOOL_FIELD_NUMBER = 6;
    private boolean defaultBool_;
    /** <code>optional bool default_bool = 6;</code> */
    public boolean getDefaultBool() {
      return defaultBool_;
    }

    public static final int DEFAULT_FLOAT_FIELD_NUMBER = 7;
    private float defaultFloat_;
    /** <code>optional float default_float = 7;</code> */
    public float getDefaultFloat() {
      return defaultFloat_;
    }

    public static final int DEFAULT_DOUBLE_FIELD_NUMBER = 8;
    private double defaultDouble_;
    /** <code>optional double default_double = 8;</code> */
    public double getDefaultDouble() {
      return defaultDouble_;
    }

    public static final int DEFAULT_BYTES_FIELD_NUMBER = 9;
    private com.google.protobuf.ByteString defaultBytes_;
    /** <code>optional bytes default_bytes = 9;</code> */
    public com.google.protobuf.ByteString getDefaultBytes() {
      return defaultBytes_;
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        byte[] data, com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        java.io.InputStream input) throws java.io.IOException {
      return PARSER.parseFrom(input);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        java.io.InputStream input, com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return PARSER.parseFrom(input, extensionRegistry);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseDelimitedFrom(
        java.io.InputStream input) throws java.io.IOException {
      return PARSER.parseDelimitedFrom(input);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseDelimitedFrom(
        java.io.InputStream input, com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return PARSER.parseDelimitedFrom(input, extensionRegistry);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        com.google.protobuf.CodedInputStream input) throws java.io.IOException {
      return PARSER.parseFrom(input);
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return PARSER.parseFrom(input, extensionRegistry);
    }

    public static Builder newBuilder() {
      return new Builder();
    }

    public Builder newBuilderForType() {
      return newBuilder();
    }

    public static Builder newBuilder(
        com.google.protobuf.jruby.SentinelOuterClass.Sentinel prototype) {
      return newBuilder().mergeFrom(prototype);
    }

    public Builder toBuilder() {
      return newBuilder(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(com.google.protobuf.GeneratedMessage.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /** Protobuf type {@code com.google.protobuf.jruby.Sentinel} */
    public static final class Builder extends com.google.protobuf.GeneratedMessage.Builder<Builder>
        implements
        // @@protoc_insertion_point(builder_implements:com.google.protobuf.jruby.Sentinel)
        com.google.protobuf.jruby.SentinelOuterClass.SentinelOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor getDescriptor() {
        return com.google.protobuf.jruby.SentinelOuterClass
            .internal_static_com_google_protobuf_jruby_Sentinel_descriptor;
      }

      protected com.google.protobuf.GeneratedMessage.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.google.protobuf.jruby.SentinelOuterClass
            .internal_static_com_google_protobuf_jruby_Sentinel_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.google.protobuf.jruby.SentinelOuterClass.Sentinel.class,
                com.google.protobuf.jruby.SentinelOuterClass.Sentinel.Builder.class);
      }

      // Construct using com.google.protobuf.jruby.SentinelOuterClass.Sentinel.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(com.google.protobuf.GeneratedMessage.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }

      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessage.alwaysUseFieldBuilders) {}
      }

      public Builder clear() {
        super.clear();
        defaultInt32_ = 0;

        defaultInt64_ = 0L;

        defaultUnit32_ = 0;

        defaultUint64_ = 0L;

        defaultString_ = "";

        defaultBool_ = false;

        defaultFloat_ = 0F;

        defaultDouble_ = 0D;

        defaultBytes_ = com.google.protobuf.ByteString.EMPTY;

        return this;
      }

      public com.google.protobuf.Descriptors.Descriptor getDescriptorForType() {
        return com.google.protobuf.jruby.SentinelOuterClass
            .internal_static_com_google_protobuf_jruby_Sentinel_descriptor;
      }

      public com.google.protobuf.jruby.SentinelOuterClass.Sentinel getDefaultInstanceForType() {
        return com.google.protobuf.jruby.SentinelOuterClass.Sentinel.getDefaultInstance();
      }

      public com.google.protobuf.jruby.SentinelOuterClass.Sentinel build() {
        com.google.protobuf.jruby.SentinelOuterClass.Sentinel result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      public com.google.protobuf.jruby.SentinelOuterClass.Sentinel buildPartial() {
        com.google.protobuf.jruby.SentinelOuterClass.Sentinel result =
            new com.google.protobuf.jruby.SentinelOuterClass.Sentinel(this);
        result.defaultInt32_ = defaultInt32_;
        result.defaultInt64_ = defaultInt64_;
        result.defaultUnit32_ = defaultUnit32_;
        result.defaultUint64_ = defaultUint64_;
        result.defaultString_ = defaultString_;
        result.defaultBool_ = defaultBool_;
        result.defaultFloat_ = defaultFloat_;
        result.defaultDouble_ = defaultDouble_;
        result.defaultBytes_ = defaultBytes_;
        onBuilt();
        return result;
      }

      private int defaultInt32_;
      /** <code>optional int32 default_int32 = 1;</code> */
      public int getDefaultInt32() {
        return defaultInt32_;
      }
      /** <code>optional int32 default_int32 = 1;</code> */
      public Builder setDefaultInt32(int value) {

        defaultInt32_ = value;
        onChanged();
        return this;
      }
      /** <code>optional int32 default_int32 = 1;</code> */
      public Builder clearDefaultInt32() {

        defaultInt32_ = 0;
        onChanged();
        return this;
      }

      private long defaultInt64_;
      /** <code>optional int64 default_int64 = 2;</code> */
      public long getDefaultInt64() {
        return defaultInt64_;
      }
      /** <code>optional int64 default_int64 = 2;</code> */
      public Builder setDefaultInt64(long value) {

        defaultInt64_ = value;
        onChanged();
        return this;
      }
      /** <code>optional int64 default_int64 = 2;</code> */
      public Builder clearDefaultInt64() {

        defaultInt64_ = 0L;
        onChanged();
        return this;
      }

      private int defaultUnit32_;
      /** <code>optional uint32 default_unit32 = 3;</code> */
      public int getDefaultUnit32() {
        return defaultUnit32_;
      }
      /** <code>optional uint32 default_unit32 = 3;</code> */
      public Builder setDefaultUnit32(int value) {

        defaultUnit32_ = value;
        onChanged();
        return this;
      }
      /** <code>optional uint32 default_unit32 = 3;</code> */
      public Builder clearDefaultUnit32() {

        defaultUnit32_ = 0;
        onChanged();
        return this;
      }

      private long defaultUint64_;
      /** <code>optional uint64 default_uint64 = 4;</code> */
      public long getDefaultUint64() {
        return defaultUint64_;
      }
      /** <code>optional uint64 default_uint64 = 4;</code> */
      public Builder setDefaultUint64(long value) {

        defaultUint64_ = value;
        onChanged();
        return this;
      }
      /** <code>optional uint64 default_uint64 = 4;</code> */
      public Builder clearDefaultUint64() {

        defaultUint64_ = 0L;
        onChanged();
        return this;
      }

      private java.lang.Object defaultString_ = "";
      /** <code>optional string default_string = 5;</code> */
      public java.lang.String getDefaultString() {
        java.lang.Object ref = defaultString_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs = (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          if (bs.isValidUtf8()) {
            defaultString_ = s;
          }
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /** <code>optional string default_string = 5;</code> */
      public com.google.protobuf.ByteString getDefaultStringBytes() {
        java.lang.Object ref = defaultString_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b =
              com.google.protobuf.ByteString.copyFromUtf8((java.lang.String) ref);
          defaultString_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /** <code>optional string default_string = 5;</code> */
      public Builder setDefaultString(java.lang.String value) {
        if (value == null) {
          throw new NullPointerException();
        }

        defaultString_ = value;
        onChanged();
        return this;
      }
      /** <code>optional string default_string = 5;</code> */
      public Builder clearDefaultString() {

        defaultString_ = getDefaultInstance().getDefaultString();
        onChanged();
        return this;
      }
      /** <code>optional string default_string = 5;</code> */
      public Builder setDefaultStringBytes(com.google.protobuf.ByteString value) {
        if (value == null) {
          throw new NullPointerException();
        }

        defaultString_ = value;
        onChanged();
        return this;
      }

      private boolean defaultBool_;
      /** <code>optional bool default_bool = 6;</code> */
      public boolean getDefaultBool() {
        return defaultBool_;
      }
      /** <code>optional bool default_bool = 6;</code> */
      public Builder setDefaultBool(boolean value) {

        defaultBool_ = value;
        onChanged();
        return this;
      }
      /** <code>optional bool default_bool = 6;</code> */
      public Builder clearDefaultBool() {

        defaultBool_ = false;
        onChanged();
        return this;
      }

      private float defaultFloat_;
      /** <code>optional float default_float = 7;</code> */
      public float getDefaultFloat() {
        return defaultFloat_;
      }
      /** <code>optional float default_float = 7;</code> */
      public Builder setDefaultFloat(float value) {

        defaultFloat_ = value;
        onChanged();
        return this;
      }
      /** <code>optional float default_float = 7;</code> */
      public Builder clearDefaultFloat() {

        defaultFloat_ = 0F;
        onChanged();
        return this;
      }

      private double defaultDouble_;
      /** <code>optional double default_double = 8;</code> */
      public double getDefaultDouble() {
        return defaultDouble_;
      }
      /** <code>optional double default_double = 8;</code> */
      public Builder setDefaultDouble(double value) {

        defaultDouble_ = value;
        onChanged();
        return this;
      }
      /** <code>optional double default_double = 8;</code> */
      public Builder clearDefaultDouble() {

        defaultDouble_ = 0D;
        onChanged();
        return this;
      }

      private com.google.protobuf.ByteString defaultBytes_ = com.google.protobuf.ByteString.EMPTY;
      /** <code>optional bytes default_bytes = 9;</code> */
      public com.google.protobuf.ByteString getDefaultBytes() {
        return defaultBytes_;
      }
      /** <code>optional bytes default_bytes = 9;</code> */
      public Builder setDefaultBytes(com.google.protobuf.ByteString value) {
        if (value == null) {
          throw new NullPointerException();
        }

        defaultBytes_ = value;
        onChanged();
        return this;
      }
      /** <code>optional bytes default_bytes = 9;</code> */
      public Builder clearDefaultBytes() {

        defaultBytes_ = getDefaultInstance().getDefaultBytes();
        onChanged();
        return this;
      }

      public final Builder setUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return this;
      }

      public final Builder mergeUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return this;
      }

      // @@protoc_insertion_point(builder_scope:com.google.protobuf.jruby.Sentinel)
    }

    // @@protoc_insertion_point(class_scope:com.google.protobuf.jruby.Sentinel)
    private static final com.google.protobuf.jruby.SentinelOuterClass.Sentinel defaultInstance;

    static {
      defaultInstance = new com.google.protobuf.jruby.SentinelOuterClass.Sentinel();
    }

    public static com.google.protobuf.jruby.SentinelOuterClass.Sentinel getDefaultInstance() {
      return defaultInstance;
    }

    public com.google.protobuf.jruby.SentinelOuterClass.Sentinel getDefaultInstanceForType() {
      return defaultInstance;
    }
  }

  private static final com.google.protobuf.Descriptors.Descriptor
      internal_static_com_google_protobuf_jruby_Sentinel_descriptor;
  private static com.google.protobuf.GeneratedMessage.FieldAccessorTable
      internal_static_com_google_protobuf_jruby_Sentinel_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor getDescriptor() {
    return descriptor;
  }

  private static com.google.protobuf.Descriptors.FileDescriptor descriptor;

  static {
    java.lang.String[] descriptorData = {
      "\n\016sentinel.proto\022\031com.google.protobuf.jr"
          + "uby\"\334\001\n\010Sentinel\022\025\n\rdefault_int32\030\001 \001(\005\022"
          + "\025\n\rdefault_int64\030\002 \001(\003\022\026\n\016default_unit32"
          + "\030\003 \001(\r\022\026\n\016default_uint64\030\004 \001(\004\022\026\n\016defaul"
          + "t_string\030\005 \001(\t\022\024\n\014default_bool\030\006 \001(\010\022\025\n\r"
          + "default_float\030\007 \001(\002\022\026\n\016default_double\030\010 "
          + "\001(\001\022\025\n\rdefault_bytes\030\t \001(\014B\002H\002b\006proto3"
    };
    com.google.protobuf.Descriptors.FileDescriptor.InternalDescriptorAssigner assigner =
        new com.google.protobuf.Descriptors.FileDescriptor.InternalDescriptorAssigner() {
          public com.google.protobuf.ExtensionRegistry assignDescriptors(
              com.google.protobuf.Descriptors.FileDescriptor root) {
            descriptor = root;
            return null;
          }
        };
    com.google.protobuf.Descriptors.FileDescriptor.internalBuildGeneratedFileFrom(
        descriptorData, new com.google.protobuf.Descriptors.FileDescriptor[] {}, assigner);
    internal_static_com_google_protobuf_jruby_Sentinel_descriptor =
        getDescriptor().getMessageTypes().get(0);
    internal_static_com_google_protobuf_jruby_Sentinel_fieldAccessorTable =
        new com.google.protobuf.GeneratedMessage.FieldAccessorTable(
            internal_static_com_google_protobuf_jruby_Sentinel_descriptor,
            new java.lang.String[] {
              "DefaultInt32",
              "DefaultInt64",
              "DefaultUnit32",
              "DefaultUint64",
              "DefaultString",
              "DefaultBool",
              "DefaultFloat",
              "DefaultDouble",
              "DefaultBytes",
            });
  }

  // @@protoc_insertion_point(outer_class_scope)
}
