// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Feng Xiao (xiaofeng@google.com)
//
// Test that custom options defined in a proto file's dependencies are properly
// initialized.
syntax = "proto2";

package protobuf_unittest;

import "google/protobuf/unittest_custom_options.proto";


message TestMessageWithCustomOptionsContainer {
  optional TestMessageWithCustomOptions field = 1;
}
