// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#ifndef UPB_MESSAGE_TYPES_H_
#define UPB_MESSAGE_TYPES_H_

// This typedef is in a leaf header to resolve a circular dependency between
// messages and mini tables.
typedef void upb_Message;

#endif /* UPB_MESSAGE_TYPES_H_ */
