// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// This file exists because of the design of the Starlark unit testing
// framework. We call these tests unittests, but Blaze still sees them as full
// builds. And while Blaze doesn't build the test-setup targets unless it really
// needs to, it checks that all input files are present. Therefore, we need
// these "empty" files to be present.

syntax = "proto2";

package third_party_protobuf_rust_test_rust_proto_library_unit_test;
