// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// These headers form a spanning tree for the upb defs needed by FFI layers.

// IWYU pragma: begin_exports
#include "upb/message/accessors_split64.h"
#include "upb/message/array_split64.h"
#include "upb/message/map.h"
#include "upb/message/message.h"
#include "upb/mini_descriptor/decode.h"
// IWYU pragma: end_exports
