syntax = "proto3";

package lower_enum_value;

enum NotAllowed {
  abstract = 0;
  and = 1;
  array = 2;
  as = 3;
  break = 4;
  callable = 5;
  case     = 6;
  catch    = 7;
  class    = 8;
  clone    = 9;
  const    = 10;
  continue = 11;
  declare  = 12;
  default  = 13;
  die      = 14;
  do       = 15;
  echo     = 16;
  else     = 17;
  elseif   = 18;
  empty    = 19;
  enddeclare = 20;
  endfor     = 21;
  endforeach = 22;
  endif      = 23;
  endswitch = 24;
  endwhile = 25;
  eval = 26;
  exit = 27;
  extends = 28;
  final = 29;
  finally = 30;
  fn = 31;
  for = 32;
  foreach = 33;
  function = 34;
  global = 35;
  goto = 36;
  if = 37;
  implements = 38;
  include = 39;
  include_once = 40;
  instanceof = 41;
  insteadof = 42;
  interface = 43;
  isset = 44;
  list = 45;
  match = 46;
  namespace = 47;
  new = 48;
  or = 49;
  parent = 77;
  print = 50;
  private = 51;
  protected = 52;
  public = 53;
  readonly = 79;
  require = 54;
  require_once = 55;
  return = 56;
  self = 78;
  static = 57;
  switch = 58;
  throw = 59;
  trait = 60;
  try = 61;
  unset = 62;
  use = 63;
  var = 64;
  while = 65;
  xor = 66;
  yield = 67;
  int = 68;
  float = 69;
  bool = 70;
  string = 71;
  true = 72;
  false = 73;
  null = 74;
  void = 75;
  iterable = 76;
}
