// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Darick Tong (darick@google.com)
//
// A proto file with nested extensions. Note that this must be defined in
// a separate file to properly test the initialization of the outer class.

syntax = "proto2";

package protobuf_unittest;

import "com/google/protobuf/non_nested_extension.proto";


message MyNestedExtension {
  extend MessageToBeExtended {
    optional MessageToBeExtended recursiveExtension = 2;
    optional int32 default = 2002;
  }
}
