// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Darick Tong (darick@google.com)
//
// A proto file with extensions.

syntax = "proto2";

package protobuf_unittest;


message MessageToBeExtended {
  extensions 1 to max;
}

message MyNonNestedExtension {}

extend MessageToBeExtended {
  optional MyNonNestedExtension nonNestedExtension = 1;
  optional string if = 2000;
}
