<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto_previous/test_previously_unreserved_message.proto

namespace GPBMetadata\ProtoPrevious;

class TestPreviouslyUnreservedMessage
{
    public static $is_initialized = false;

    public static function initOnce() {
        $pool = \Google\Protobuf\Internal\DescriptorPool::getGeneratedPool();

        if (static::$is_initialized == true) {
          return;
        }
        $pool->internalAddGeneratedFile(
            '
W
7proto_previous/test_previously_unreserved_message.protoprevious"

readonlybproto3'
        , true);

        static::$is_initialized = true;
    }
}

