#region Copyright notice and license
// Protocol Buffers - Google's data interchange format
// Copyright 2015 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd
#endregion

namespace Google.Protobuf
{
    // Just a sample enum with positive and negative values to be used in tests.
    internal enum SampleEnum
    {
        NegativeValue = -2,
        None = 0,
        PositiveValue = 3
    }
}
