// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: Jacob Butcher (jbaum@google.com)
//
// Test file option java_string_check_utf8.
syntax = "proto2";

package proto2_test_check_utf8_size;

option java_outer_classname = "TestCheckUtf8Size";
option java_string_check_utf8 = true;
option optimize_for = CODE_SIZE;

message StringWrapperSize {
  required string req = 1;
  optional string opt = 2;
  repeated string rep = 3;
}

message BytesWrapperSize {
  required bytes req = 1;
  optional bytes opt = 2;
  repeated bytes rep = 3;
}
