// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#ifndef UPB_MINI_TABLE_EXTENSION_H_
#define UPB_MINI_TABLE_EXTENSION_H_

#include "upb/mini_table/internal/extension.h"

typedef struct upb_MiniTableExtension upb_MiniTableExtension;

#endif /* UPB_MINI_TABLE_EXTENSION_H_ */
