// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

#include "upb/mini_table/internal/message.h"

const struct upb_MiniTable _kUpb_MiniTable_Empty = {
    .subs = NULL,
    .fields = NULL,
    .size = 0,
    .field_count = 0,
    .ext = kUpb_ExtMode_NonExtendable,
    .dense_below = 0,
    .table_mask = -1,
    .required_count = 0,
};
