// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package google.protobuf.python.internal;

message TestEnumValues {
  enum NestedEnum {
    ZERO = 0;
    ONE = 1;
  }
  optional NestedEnum optional_nested_enum = 1;
  repeated NestedEnum repeated_nested_enum = 2;
  repeated NestedEnum packed_nested_enum = 3 [packed = true];
}

message TestMissingEnumValues {
  enum NestedEnum {
    TWO = 2;
  }
  optional NestedEnum optional_nested_enum = 1;
  repeated NestedEnum repeated_nested_enum = 2;
  repeated NestedEnum packed_nested_enum = 3 [packed = true];
}

message JustString {
  optional string dummy = 1;
}
