// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: liujisi@google.com (Pherl Liu)

syntax = "proto2";

package objc.protobuf.tests.public_import;

option objc_class_prefix = "PublicImport";

message Message {
  optional int32 e = 1;
}
