// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package unittest_issues;

option csharp_namespace = "UnitTest.Issues.TestProtos";

// This file is used as part of a unit test for issue 6936
// We don't need to use it, we just have to import it in both
// "extensions_issue6936_b.proto" and "extensions_issue6936_c.proto"

import "google/protobuf/descriptor.proto";

extend google.protobuf.MessageOptions {
  string opt = 50000;
}
