// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package google.protobuf.python.internal.import_test_package;

// Test public import
import public "google/protobuf/internal/import_test_package/import_public.proto";

message Inner {
  optional int32 value = 1 [default = 57];
}
