// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package naming;

message Reserved {
  optional int32 for = 1; // for is a reserved word in rust, let's make sure we can compile it

  message Inner {}
  optional Inner pub = 2;  // pub is reserved too!
}
