// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Author: kenton@google.com (Kenton Varda)

syntax = "proto2";

package protobuf_unittest;

option py_generic_services = true;

message TestBadIdentifiers {
  extensions 100 to max;
}

// Make sure these reasonable extension names don't conflict with internal
// variables.
extend TestBadIdentifiers {
  optional string message = 100 [default = "foo"];
  optional string descriptor = 101 [default = "bar"];
  optional string reflection = 102 [default = "baz"];
  optional string service = 103 [default = "qux"];
}

message AnotherMessage {}
service AnotherService {}
