// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

// Temporary hack to fix gRPC. Do not use.

#ifndef UPB_HPP_
#define UPB_HPP_

// IWYU pragma: begin_exports
#include "upb/base/status.hpp"
#include "upb/mem/arena.hpp"
// IWYU pragma: end_exports

#endif  // UPB_HPP_
