// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package protobuf_unittest;


message TestMessage3 {
  message NestedMessage {
    // This enum's name is the same with the default outer class name of this
    // proto file. It's used to test if the compiler can avoid this conflict
    // correctly.
    enum OuterClassNameTest3 { DUMMY_VALUE = 1; }
  }
}
