// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// LINT: LEGACY_NAMES
syntax = "proto2";

package protobuf.kotlin.generator;

option java_package = "in.com.google.protobuf.kotlin.generator";
option java_multiple_files = true;

message EvilNamesProto2 {
  optional bool initialized = 1;
  optional bool has_foo = 2;
  optional string Bar = 3;
  optional bool is_initialized = 4;

  oneof camelCase {
    string fooBar = 5;
  }

  repeated string ALL_CAPS = 7;
  map<int32, bool> ALL_CAPS_MAP = 8;

  optional bool has_underbar_preceding_numeric_1foo = 9;
  optional bool has_underbar_preceding_numeric_42bar = 13;
  optional bool has_underbar_preceding_numeric_123foo42bar_baz = 14;

  extensions 100 to max;

  repeated string extension = 12;
  repeated int32 class = 15;
  optional double int = 16;
  optional bool long = 17;
  optional int64 boolean = 18;
  optional string sealed = 19;
  optional float interface = 20;
  optional string object = 22;
  optional string cached_size = 23;
  optional bool serialized_size = 24;
  optional string by = 25;
}

message List {}

message HardKeywordsAllTypesProto2 {
  message NestedMessage {
    optional int32 while = 1;
  }

  enum NestedEnum {
    FOO = 1;
    BAR = 2;
  }

  optional int32 as = 1;
  optional NestedEnum break = 3;
  map<int32, int32> continue = 4;
  optional NestedMessage do = 5;

  repeated int32 else = 6;
  repeated string for = 7;
  repeated NestedEnum fun = 8;
  repeated NestedMessage if = 9;
}

message Interface {}
