// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package dynamic_message_test;

option java_package = "dynamicmessagetest";
option java_outer_classname = "DynamicMessageTestProto";

message EmptyMessage {}

message MessageWithMapFields {
  map<string, EmptyMessage> string_message_map = 1;
}
